@@ -307,7 +307,7 @@ static void process(const int clientfd)
     char line[4096];
     
     while ((readen = read(clientfd, line, sizeof line - 1U)) < (ssize_t) 0 &&
-           errno == EINTR);
+           (errno == EINTR || errno == EIO));
     if (readen <= (ssize_t) 0) {
         return;
     }
@@ -376,15 +376,17 @@ static void process(const int clientfd)
 
 int listencnx(void)
 {
+    struct sockaddr_un *saddr;    
     int clientfd;
     int ret = -1;
-    struct sockaddr_un *saddr;
+    const size_t socketpath_len = strlen(socketpath);
         
-    if ((saddr = malloc(sizeof(*saddr) + strlen(socketpath) + 1U)) == NULL) {
+    if ((saddr = malloc(sizeof(*saddr) + socketpath_len + 
+                        (size_t) 1U)) == NULL) {
         perror("No more memory to listen to anything");
         goto bye;
     }
-    strcpy(saddr->sun_path, socketpath);   /* flawfinder: ignore - safe */
+    memcpy(saddr->sun_path, socketpath, socketpath_len + (size_t) 1U);
     saddr->sun_family = AF_UNIX;
     (void) unlink(socketpath);
     (void) umask(077);    
@@ -425,9 +427,8 @@ int listencnx(void)
         kindy = -1;
     }
     (void) unlink(socketpath);
-    if (saddr != NULL) {
-        free(saddr);
-    }
+    free(saddr);
+    
     return ret;
 }
 
